/* libexs/serial.h
 * RFC 1982 Serial number manipulation functions for EXS.
 */

/*
 * Copyright 2013, 2014 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef LIBEXS_SERIAL_H
#define LIBEXS_SERIAL_H

#include <stdint.h>

static const uint32_t partition_32 = UINT32_C(1) << 31;
static const uint64_t partition_64 = UINT64_C(1) << 63;


/* Returns true if x is less than y by serial number arithmetic. */
static inline int
serial_less(uint32_t x, uint32_t y)
{
	return (x != y) && (((x < y) && (y - x < partition_32))
		|| ((y < x) && (x - y > partition_32)));
}	/* serial_less */

static inline int
serial_less_64(uint64_t x, uint64_t y)
{
	return (x != y) && (((x < y) && (y - x < partition_64))
		|| ((y < x) && (x - y > partition_64)));
}	/* serial_less_64 */


/* Returns true if x is greater than y by serial number arithmetic. */
static inline int
serial_greater(uint32_t x, uint32_t y)
{
	return (x != y) && (((x < y) && (y - x > partition_32))
		|| ((y < x) && (x - y < partition_32)));
}	/* serial_greater */

static inline int
serial_greater_64(uint64_t x, uint64_t y)
{
	return (x != y) && (((x < y) && (y - x > partition_64))
		|| ((y < x) && (x - y < partition_64)));
}	/* serial_greater_64 */


#endif
